unit SmtpMailSS;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs, NMsmtp, Logging;

type
  TSmtpMailSS = class(TWinControl)
    constructor Create;
    destructor Destroy;
    procedure NMSMTP1AttachmentNotFound(Filename: String);
    procedure NMSMTP1AuthenticationFailed(var Handled: Boolean);
    procedure NMSMTP1Connect(Sender: TObject);
    procedure NMSMTP1ConnectionFailed(Sender: TObject);
    procedure NMSMTP1ConnectionRequired(var Handled: Boolean);
    procedure NMSMTP1Disconnect(Sender: TObject);
    procedure NMSMTP1EncodeEnd(Filename: String);
    procedure NMSMTP1EncodeStart(Filename: String);
    procedure NMSMTP1Failure(Sender: TObject);
    procedure NMSMTP1HeaderIncomplete(var handled: Boolean;
         hiType: Integer);
    procedure NMSMTP1HostResolved(Sender: TComponent);
    procedure NMSMTP1InvalidHost(var Handled: Boolean);
    procedure NMSMTP1MailListReturn(MailAddress: String);
    procedure NMSMTP1PacketSent(Sender: TObject);
    procedure NMSMTP1RecipientNotFound(Recipient: String);
    procedure NMSMTP1SendStart(Sender: TObject);
    procedure NMSMTP1Status(Sender: TComponent; Status: String);
    procedure NMSMTP1Success(Sender: TObject);
  private
    Lines: TStringlist;
    Attachments: TStringlist;
    CCRecipients: TStringlist;
    Host: String;
    Port: String;
    Subject: String;
    FromAdress: String;
    ToAdress: String;
    UserID: String;
    Logging: TLogging;
    IsDebugging: Boolean;
    Success: Integer;
    _LastError: String;
    procedure DebugMessage(aMessage: String);
  protected
  public
    procedure NewMessage(aFromAdress: String; aSubject: String);
    procedure AddLine(aLine: String);
    function Send: Boolean;
    procedure SetToAdress(aToAdress: String);
    procedure SetFromAdress(aFromAdress: String);
    procedure SetSubject(aSubject: String);
    procedure AddAttachment(anAttachment: String);
    procedure AddCCRecipients(aCCRecipient: String);
    procedure SetHost(aHost: String);
    procedure SetUserID(anUserID: String);
    procedure ClearAttachments;
    procedure ClearLines;
    function LastError: String;
    function WasSuccessful: Boolean;
    procedure SetDebugMode;
    procedure SetWorkMode;
  published
    { Published-Deklarationen }
  end;

procedure Register;

implementation

procedure Register;
begin
  RegisterComponents('Beispiele', [TSmtpMailSS]);
end;


constructor TSmtpMailSS.Create;
begin
  // create the sub objects and initialize certain fields
  Lines:=TStringlist.Create;
  Lines.Clear;
  Attachments:=TStringlist.Create;
  Attachments.Clear;
  CCRecipients:=TStringlist.Create;
  CCRecipients.Clear;
  Host:='';
  Port:='25';
  Subject:='';
  FromAdress:='';
end;


destructor TSmtpMailSS.Destroy;
begin
  // delete the sub objects initialized in the constructor
  Lines.Free;
  Attachments.Free;
  CCRecipients.Free;
end;


procedure TSmtpMailSS.NewMessage;
begin
  // create a new message
  // check if all sub objects have been initialized
  // (just to be sure) and store the parameter values given when
  // calling this function within local member variables
  if Lines<>nil then Lines.Free;
  Lines:=TStringlist.Create;
  Lines.Clear;

  if Attachments<>nil then Attachments.Free;
  Attachments:=TStringlist.Create;
  Attachments.Clear;

  if CCRecipients<>nil then CCRecipients.Free;
  CCRecipients:=TStringlist.Create;
  CCRecipients.Clear;

  Host:='';
  Port:='25';
  IsDebugging:=FALSE;

  FromAdress:=aFromAdress;
  Subject:=aSubject;
end;


function TSmtpMailSS.Send;
var smtp: TNMSMTP;
begin
  // hope the best (that the sending process will succeed)
  success:=1;
  _LastError:='';

  // create the logging object
  // (needed for debugging purposes)
  Logging:=TLogging.Create;
  Logging.SetFileSuffix('smtp');
  Logging.SetDirectory('c:\temp');

  // check if all values have been previously set
  // if not, jump out
  result:=FALSE;
  if Length(host)=0 then exit;
  if Length(FromAdress)=0 then exit;
  if Length(ToAdress)=0 then exit;
  if Lines.Count<1 then exit;
  result:=TRUE;

  // create the smtp object, data type is the TNMSTP
  // component class which ships with Delphi
  smtp:=TNMSMTP.Create(self);
  smtp.host:=host;

  // set the various fields of the component
  smtp.PostMessage.Attachments.Clear;
  if Attachments<>nil then
    if Attachments.Count>0 then
      smtp.PostMessage.Attachments.Assign(Attachments);

  DebugMessage('Nach Attachment-Zuweisung');

  smtp.PostMessage.FromAddress:=FromAdress;
//  DebugMessage('FromAdress:'+FromAdress);
  smtp.PostMessage.ToAddress.Clear;
  smtp.PostMessage.ToAddress.Add(ToAdress);
//  DebugMessage('ToAdress:'+ToAdress);
  smtp.PostMessage.Subject:=Subject;
//  DebugMessage('Subject:'+Subject);
  smtp.UserID:=UserID;

  smtp.PostMessage.ToCarbonCopy:=TStringList.Create;
  smtp.PostMessage.ToCarbonCopy.Assign(CCRecipients);

  // set the mail body
  smtp.PostMessage.Body.Clear;
  if Lines<>nil then
    if Lines.Count>0 then
      smtp.PostMessage.Body.Assign(Lines);


  // start the send process
  if smtp.Connected then
    smtp.Disconnect;
  smtp.Connect;
  DebugMessage('SMTP-Objekt Connect');
  smtp.SendMail;
  DebugMessage('SMTP-Objekt SendMail');
  smtp.Disconnect;
  DebugMessage('SMTP-Objekt Disconnect');
  smtp.Free;
  DebugMessage('SMTP-Objekt destroyed');

  // delete the sub objects previously initialized
  if Logging<>nil then
    Logging.Free;
end;


function TSmtpMailSS.LastError: String;
begin
  // for error checking within Navision Financials
  result:=_LastError;
end;


function TSmtpMailSS.WasSuccessful: Boolean;
begin
  // for error checking within Navision Financials
  if success=0 then result:=FALSE
    else result:=TRUE;
end;


procedure TSmtpMailSS.DebugMessage;
begin
  // function which writes certain entries into a
  // logging file which is creating by the
  // Logging-object. This is done only if the component
  // is in debug mode
  if not IsDebugging then exit;

  if Logging=nil then begin
    Logging:=TLogging.Create;
    Logging.SetFileSuffix('smtp');
    Logging.SetDirectory('c:\temp');
  end;
  Logging.AddMessage(aMessage);
end;


procedure TSmtpMailSS.SetDebugMode;
begin
  // activate debug mode
  IsDebugging:=TRUE;
end;


procedure TSmtpMailSS.SetWorkMode;
begin
  // deactive debug mode
  IsDebugging:=FALSE;
end;


// ----------------------------------------------------
// ------------- Property methods ---------------------
procedure TSmtpMailSS.SetToAdress(aToAdress: String);
begin
  ToAdress:=aToAdress;
end;


procedure TSmtpMailSS.SetFromAdress(aFromAdress: String);
begin
  FromAdress:=aFromAdress;
end;


procedure TSmtpMailSS.SetSubject(aSubject: String);
begin
  Subject:=aSubject;
end;


procedure TSmtpMailSS.AddAttachment;
begin
  if Attachments=nil then Attachments:=TStringlist.Create;
  Attachments.Append(anAttachment);
end;


procedure TSmtpMailSS.ClearAttachments;
begin
  Attachments.Clear;
end;


procedure TSmtpMailSS.ClearLines;
begin
  Lines.Clear;
end;

procedure TSmtpMailSS.SetHost(aHost: String);
begin
  Host:=aHost;
end;


procedure TSmtpMailSS.AddCCRecipients(aCCRecipient: String);
begin
  if CCRecipients=nil then CCRecipients:=TStringlist.Create;
  CCRecipients.Append(aCCRecipient);
end;


procedure TSmtpMailSS.SetUserID(anUserID: String);
begin
  UserID:=anUserID;
end;


procedure TSmtpMailSS.AddLine;
begin
  if Lines=nil then Lines:=TStringlist.Create;
  Lines.Add(aLine);
end;





// --------------------------------------------------------------
// --------------- to maintain control of the sub component state
// --------------------------------------------------------------
procedure TSmtpMailSS.NMSMTP1HostResolved(Sender: TComponent);
begin
  DebugMessage('SMTP Host Resolved');
end;

procedure TSmtpMailSS.NMSMTP1InvalidHost(var Handled: Boolean);
begin
  DebugMessage('SMTP Invalid Host');
  _LastError:='SMTP Invalid Host';
  success:=0;
end;

procedure TSmtpMailSS.NMSMTP1MailListReturn(MailAddress: String);
begin
  DebugMessage('SMTP MailList Return');
  _LastError:='SMTP MailList Return';
end;

procedure TSmtpMailSS.NMSMTP1PacketSent(Sender: TObject);
begin
  DebugMessage('SMTP Packet Sent');
end;

procedure TSmtpMailSS.NMSMTP1RecipientNotFound(Recipient: String);
begin
  DebugMessage('SMTP Recipient Not Found');
  _LastError:='SMTP Recipient Not Found';
end;

procedure TSmtpMailSS.NMSMTP1SendStart(Sender: TObject);
begin
  DebugMessage('SMTP Send Start');
end;

procedure TSmtpMailSS.NMSMTP1Status(Sender: TComponent; Status: String);
begin
  DebugMessage('SMTP Status');
end;

procedure TSmtpMailSS.NMSMTP1Success(Sender: TObject);
begin
  DebugMessage('SMTP Success');
  success:=1;
end;

procedure TSmtpMailSS.NMSMTP1HeaderIncomplete(var handled: Boolean;
  hiType: Integer);
begin
  DebugMessage('SMTP Header Incomplete');
  _LastError:='SMTP Header Incomplete';
end;

procedure TSmtpMailSS.NMSMTP1Failure(Sender: TObject);
begin
  DebugMessage('SMTP Failure');
  _LastError:='SMTP Failure';
end;

procedure TSmtpMailSS.NMSMTP1EncodeStart(Filename: String);
begin
  DebugMessage('SMTP Encode Start');
end;

procedure TSmtpMailSS.NMSMTP1EncodeEnd(Filename: String);
begin
  DebugMessage('SMTP Encode End');
end;

procedure TSmtpMailSS.NMSMTP1ConnectionRequired(var Handled: Boolean);
begin
  DebugMessage('SMTP Connection Required');
  success:=0;
  _LastError:='SMTP Connection Required';
end;

procedure TSmtpMailSS.NMSMTP1Disconnect(Sender: TObject);
begin
  DebugMessage('SMTP Disconnect');
end;

procedure TSmtpMailSS.NMSMTP1ConnectionFailed(Sender: TObject);
begin
  DebugMessage('SMTP Connection Failed');
  _LastError:='SMTP Connection Failed';
  success:=0;
end;

procedure TSmtpMailSS.NMSMTP1Connect(Sender: TObject);
begin
  DebugMessage('SMTP Connect');
end;

procedure TSmtpMailSS.NMSMTP1AuthenticationFailed(var Handled: Boolean);
begin
  DebugMessage('SMTP Authentication Failed');
  _LastError:='SMTP Authentication Failed';
  success:=0;
end;

procedure TSmtpMailSS.NMSMTP1AttachmentNotFound(Filename: String);
begin
  DebugMessage('SMTP Attachment Not Found');
  _LastError:='SMTP Attachment Not Found';
  success:=0;
end;

end.
